/*
*******************************************************************************
**
**  This device driver was created by Applilet2 for 78K0/Kx2
**  8-Bit Single-Chip Microcontrollers
**
**  Copyright(C) NEC Electronics Corporation 2002 - 2011
**  All rights reserved by NEC Electronics Corporation.
**
**  This program should be used on your own responsibility.
**  NEC Electronics Corporation assumes no responsibility for any losses
**  incurred by customers or third parties arising from the use of this file.
**
**  Filename :	Ad.c
**  Abstract :	This file implements device driver for AD module.
**  APIlib :	Applilet2 for 78K0/Kx2 V2.41 [29 Apr. 2008]
**
**  Device :	uPD78F0512_48
**
**  Compiler :	CC78K0
**
**  Creation date:	2011/12/23
**  
*******************************************************************************
*/

/*
*******************************************************************************
**  Include files
*******************************************************************************
*/
#include "macrodriver.h"
#include "Ad.h"
/* Start user code for include definition. Do not edit comment generated here */
/* End user code for include definition. Do not edit comment generated here */
#include "user_define.h"

/*
*******************************************************************************
**  Global define
*******************************************************************************
*/
/* Start user code for global definition. Do not edit comment generated here */
/* End user code for global definition. Do not edit comment generated here */


/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function initializes the A/D converter.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void AD_Init( void )
{
	ADM = AD_ADM_INITIALVALUE;	/* AD conversion disable */
	ADMK = 1;		/* INTAD disabled */
	ADIF = 0;		/* INTAD interrupt flag clear */
	ADPC = AD_ADPC_1ANALOG;
	/* AD pin setting */
	/* set 1 analog input */
	PM2 |= 0x80;
	ADM |= AD_CONVERSION_TIME0|AD_CONVERSION_AVREF_23v;
	ADS = AD_INPUT_CHANNEL_7;
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function starts the A/D converter.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void AD_Start( void )
{
	ADCS = 1;		/* AD conversion enable */
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function returns the conversion result in the buffer.
**
**	Parameters:
**		*buffer :	the address where to write the conversion result
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void AD_Read( USHORT* buffer )
{
	*buffer = (USHORT)( ADCR >> 6 );
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function stops the A/D converter.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void AD_Stop( void )
{
	ADCS = 0;		/* AD conversion disable */
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function enables comparator operation.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void AD_ComparatorOn( void )
{
	ADCE = 1;		/* AD comparator enable */
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function stops comparator operation.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void AD_ComparatorOff( void )
{
	ADCE = 0;		/* AD comparator disable */
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function selects analog input channel.
**
**	Parameters:
**		channel :	analog input channel
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void AD_SelectADChannel( enum ADChannel channel )
{
	ADS = (UCHAR)channel;
}

/* Start adding user code. Do not edit comment generated here */
/* End user code adding. Do not edit comment generated here */


